/*  miscc -- a compiler for misc
 *    Tree traverser.
 *
 *    Michel Schinz
 *    $Id: Traverser.java,v 1.2 2004/03/23 09:38:47 schinz Exp $
 */

package miscc;

import miscc.Tree.*;

/**
 * Visitor traversing the tree without doing any modification to it.
 */

public class Traverser implements Visitor {
    public void traverse(Tree tree) {
        tree.apply(this);
    }

    private void traverse(Tree[] trees) {
        for (int i = 0; i < trees.length; ++i)
            traverse(trees[i]);
    }

    public void caseBad(Bad tree) {
        /* nothing to do */
    }

    public void caseProgram(Program tree) {
        traverse(tree.defs);
        traverse(tree.body);
    }

    public void caseBody(Body tree) {
        traverse(tree.stats);
        traverse(tree.expr);
    }

    public void caseFunDef(FunDef tree) {
        traverse(tree.body);
    }

    public void caseFormal(Formal tree) {
        traverse(tree.vartype);
    }

    public void caseBasicType(BasicType tree) {
        /* nothing to do */
    }

    public void caseListType(ListType tree) {
        traverse(tree.elemtype);
    }

    public void caseFunType(FunType tree) {
        traverse(tree.argtypes);
        traverse(tree.restype);
    }

    public void caseVarDef(VarDef tree) {
        traverse(tree.init);
    }

    public void caseWhile(While tree) {
        traverse(tree.cond);
        traverse(tree.body);
    }

    public void caseExec(Exec tree) {
        traverse(tree.expr);
    }

    public void caseIf(If tree) {
        traverse(tree.cond);
        traverse(tree.thenp);
        traverse(tree.elsep);
    }

    public void caseAssign(Assign tree) {
        traverse(tree.expr);
    }

    public void caseOperation(Operation tree) {
        traverse(tree.left);
        if (tree.right != null)
            traverse(tree.right);
    }

    public void caseListop(Listop tree) {
        traverse(tree.args);
    }

    public void caseIdent(Ident tree) {
        /* nothing to do */
    }

    public void caseUnitLit(UnitLit tree) {
        /* nothing to do */
    }

    public void caseNilLit(NilLit tree) {
        /* nothing to do */
    }

    public void caseIntLit(IntLit tree) {
        /* nothing to do */
    }

    public void caseApply(Apply tree) {
        traverse(tree.fun);
        traverse(tree.args);
    }
}
